@@ -39,8 +39,10 @@ impl PostgresDriver {
     pub fn new() -> Self {
         PostgresDriver {}
     }
+}
 
-    pub fn connect(&self, url: &str) -> rdbc::Result<Rc<RefCell<dyn rdbc::Connection>>> {
+impl rdbc::Driver for PostgresDriver {
+    fn connect(&self, url: &str) -> rdbc::Result<Rc<RefCell<dyn rdbc::Connection + 'static>>> {
         postgres::Connection::connect(url, TlsMode::None)
             .map_err(|e| to_rdbc_err(&e))
             .map(|c| {
@@ -105,7 +107,7 @@ struct PStatement<'a> {
 impl<'a> rdbc::Statement for PStatement<'a> {
     fn execute_query(
         &mut self,
-        params: &Vec<rdbc::Value>,
+        params: &[rdbc::Value],
     ) -> rdbc::Result<Rc<RefCell<dyn rdbc::ResultSet + '_>>> {
         let params = to_postgres_value(params);
         let params: Vec<&dyn postgres::types::ToSql> = params.iter().map(|v| v.as_ref()).collect();
@@ -124,7 +126,7 @@ impl<'a> rdbc::Statement for PStatement<'a> {
             })
     }
 
-    fn execute_update(&mut self, params: &Vec<rdbc::Value>) -> rdbc::Result<u64> {
+    fn execute_update(&mut self, params: &[rdbc::Value]) -> rdbc::Result<u64> {
         let params = to_postgres_value(params);
         let params: Vec<&dyn postgres::types::ToSql> = params.iter().map(|v| v.as_ref()).collect();
         self.conn
@@ -153,12 +155,36 @@ impl rdbc::ResultSet for PResultSet {
         }
     }
 
-    fn get_i32(&self, i: u64) -> Option<i32> {
-        self.rows.get(self.i - 1).get(i as usize - 1)
+    fn get_i8(&self, i: u64) -> rdbc::Result<Option<i8>> {
+        unimplemented!()
     }
 
-    fn get_string(&self, i: u64) -> Option<String> {
-        self.rows.get(self.i - 1).get(i as usize - 1)
+    fn get_i16(&self, i: u64) -> rdbc::Result<Option<i16>> {
+        unimplemented!()
+    }
+
+    fn get_i32(&self, i: u64) -> rdbc::Result<Option<i32>> {
+        Ok(self.rows.get(self.i - 1).get(i as usize))
+    }
+
+    fn get_i64(&self, i: u64) -> rdbc::Result<Option<i64>> {
+        unimplemented!()
+    }
+
+    fn get_f32(&self, i: u64) -> rdbc::Result<Option<f32>> {
+        unimplemented!()
+    }
+
+    fn get_f64(&self, i: u64) -> rdbc::Result<Option<f64>> {
+        unimplemented!()
+    }
+
+    fn get_string(&self, i: u64) -> rdbc::Result<Option<String>> {
+        Ok(self.rows.get(self.i - 1).get(i as usize))
+    }
+
+    fn get_bytes(&self, i: u64) -> rdbc::Result<Option<Vec<u8>>> {
+        unimplemented!()
     }
 }
 
@@ -171,11 +197,11 @@ fn to_rdbc_type(ty: &Type) -> rdbc::DataType {
     match ty.name() {
         "" => rdbc::DataType::Bool,
         //TODO all types
-        _ => rdbc::DataType::Varchar,
+        _ => rdbc::DataType::Utf8,
     }
 }
 
-fn to_postgres_value(values: &Vec<rdbc::Value>) -> Vec<Box<dyn postgres::types::ToSql>> {
+fn to_postgres_value(values: &[rdbc::Value]) -> Vec<Box<dyn postgres::types::ToSql>> {
     values
         .iter()
         .map(|v| match v {
@@ -191,6 +217,7 @@ fn to_postgres_value(values: &Vec<rdbc::Value>) -> Vec<Box<dyn postgres::types::
 mod tests {
 
     use super::*;
+    use std::sync::Arc;
 
     #[test]
     fn execute_query() -> rdbc::Result<()> {
@@ -201,7 +228,7 @@ mod tests {
             &vec![rdbc::Value::Int32(123)],
         )?;
 
-        let driver = PostgresDriver::new();
+        let driver: Arc<dyn rdbc::Driver> = Arc::new(PostgresDriver::new());
         let conn = driver.connect("postgres://rdbc:secret@127.0.0.1:5433")?;
         let mut conn = conn.as_ref().borrow_mut();
         let stmt = conn.prepare("SELECT a FROM test")?;
@@ -211,7 +238,7 @@ mod tests {
         let mut rs = rs.as_ref().borrow_mut();
 
         assert!(rs.next());
-        assert_eq!(Some(123), rs.get_i32(1));
+        assert_eq!(Some(123), rs.get_i32(0).unwrap());
         assert!(!rs.next());
 
         Ok(())
@@ -219,7 +246,7 @@ mod tests {
 
     fn execute(sql: &str, values: &Vec<rdbc::Value>) -> rdbc::Result<u64> {
         println!("Executing '{}' with {} params", sql, values.len());
-        let driver = PostgresDriver::new();
+        let driver: Arc<dyn rdbc::Driver> = Arc::new(PostgresDriver::new());
         let conn = driver.connect("postgres://rdbc:secret@127.0.0.1:5433")?;
         let mut conn = conn.as_ref().borrow_mut();
         let stmt = conn.prepare(sql)?;
